﻿local Waterfall = AceLibrary:HasInstance("Waterfall-1.0") and AceLibrary("Waterfall-1.0");

BeneCast.GetSnapToOption = function()

	if BeneCastSetup.SnapTo == nil then
		BeneCastSetup.SnapTo = 'BeneCast';
	end
	return BeneCast.SnapTo[BeneCastSetup.SnapTo].name;
	
end

BeneCast.SetSnapToOption = function(addon_name)

	for addonname, addon in pairs(wv.SnapToFound) do
		if addon.name == addon_name then
			BeneCastSetup.SnapTo = addonname;
			BeneCast.SnapTo:ApplyConfigTable();
			BeneCast.SnapTo:SnapPanels();
			do break end;
		end
	end
	
end

BeneCast.GenerateSnapToOptions = function()

	local snaptooptions = {};
	for addonname, addon in pairs(wv.SnapToFound) do
		table.insert(snaptooptions,addon.name);
	end
	return snaptooptions;
	
end

BeneCast.GetSetupOption = function(info)

	return BeneCastSetup[info[#info]];
	
end

BeneCast.SetSetupOption = function(info,value)

	BeneCastSetup[info[#info]]=value;
	BeneCast:Print(info[#info] .. " was set to: " .. tostring(value) );

end

BeneCast.GetProfileSettingsOption = function(info)

	return BeneCastProfiles[info[#info-2]][info[#info]];
	
end

BeneCast.SetProfileSettingsOption = function(info, value)

	local pfid = info[#info-2];
	local option = info[#info];
	BeneCastProfiles[pfid][option] = value;
	if option == 'buttonsize' then
		BeneCast:ResizeButtonsForProfile(pfid);
	
	elseif option == 'flashasfade' or option == 'showallbuttons' then
		BeneCast:UpdatePanelsForProfile(pfid);
		
	elseif option == 'checkrange' then
		if value then
			BeneCast.Timers.CheckRange:Enable();
		else
			BeneCast.Timers.CheckRange:Disable();
		end
	
	elseif option == 'showpets' then
		-- Attach the panels again, just to be sure
		self.SnapTo:SnapPanels();
		
		for pet, owner in pairs(BeneCast.Constants.OwnersByPet) do
			BeneCast:UpdatePanelsForUnit(pet);
		end
		
	elseif option == 'showtargettarget' then
		-- Attach the panels again, just to be sure
		self.SnapTo:SnapPanels();
		
		-- Redraw the buttons for the target of target
		self:UpdatePanelsForUnit('targettarget');
	
	end
		
	BeneCast:Print(info[#info] .. " was set to: " .. tostring(value) );

end

local defaultoptions = {
    name = 'BeneCast',
    type = 'group',
    args = {
		general = {
			type = 'group',
			name = 'general',
			desc = L['General'],
			args = {
				win = {
					type = 'execute',
					name = 'win',
					desc = L['Settings'],
					func = function()
						if Waterfall then
							Waterfall:Open('BeneCast')
						else
							self:Print(L["Waterfall-1.0 is required to access the GUI."])
						end
					end,
				},
				snapto = {
					type = 'select',
					name = 'snapto',
					desc = L['Snap To Options'],
					values = function()
						local snaptooptions = {};
						for addonname, addon in pairs(wv.SnapToFound) do
							table.insert(snaptooptions,addon.name);
						end
						return snaptooptions;
					end,
					get = function()
						if BeneCastSetup.snapto == nil then
							BeneCastSetup.snapto = 'BeneCast';
						end
						return BeneCast.SnapTo[BeneCastSetup.snapto].name;
					end,
					set = function(info,addon_name)
						BeneCastSetup.snapto = _GetKeyForValInMember(wv.SnapToFound,'name',addon_name);
						BeneCast.SnapTo:ApplyConfigTable();
						BeneCast.SnapTo:SnapPanels();
					end,
				},
				debug = {
					type = 'toggle',
					name = 'debug',
					desc = 'Debug',
					get = BeneCast.GetSetupOption,
					set = BeneCast.SetSetupOption,
				},
				hideminimapbutton = {
					type = 'toggle',
					name = 'hideminimapbutton',
					desc = L['Hide the minimap button'],
					get = function()
						return BeneCastSetup.hideminimapbutton;
					end,
					set = function(val)
						BeneCastSetup.hideminimapbutton = val;
						if MinimapButton then
							if val then
								MinimapButton:Hide();
							else
								MinimapButton:Show();
							end
						end
					end,
				},
			},
		},
        ooc = {
			type = 'group',
			name = 'ooc',
			desc = L['Out of Combat'],
			args = {				
				settings = {
					type = 'group',
					name = 'settings',
					desc = L['Settings'],
					args {
						orientation = {
							type = 'select',
							name = 'orientation',
							desc = L['Orientation'],
							values = function(info)
								local vals = {};
								local currentorient = 'RIGHT';
								local orientobj;
								while table.getn(vals) == 0 and currentorient ~= 'RIGHT' do
									orientobj = BeneCast.Constants.Orientations[currentorient];
									table.insert(vals, orientobj.translation);
									currentorient = orientobj.next;
								end
							end,
							get = function(info)
								return BeneCast.Constants.Orientations[BeneCastProfiles[info[2]].orientation].translation;
							end,
							set = function(info,val)
								BeneCastProfiles[info[2]].orientation = _GetKeyForValInMember(BeneCast.Constants.Orientations,'translation',val);
							end,
						},
						buttonsize = {
							type = 'range',
							name = 'buttonsize',
							desc = L['Button Size'],
							min = 8,
							max = 36,
							step = 1,
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						flashasfade = {
							type = 'toggle',
							name = 'flashasfade',
							desc = L['Flash as Fade'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						checkrange = {
							type = 'toggle',
							name = 'checkrange',
							desc = L['Check Range for Target'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						showtooltips = {
							type = 'toggle',
							name = 'showtooltips',
							desc = L['Show Button Tooltips'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						showtooltipname = {
							type = 'toggle',
							name = 'showtooltipname',
							desc = L['Spell Name Only'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						showallbuttons = {
							type = 'toggle',
							name = 'showallbuttons',
							desc = L['Always Show Buttons'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						unlockbuttons = {
							type = 'toggle',
							name = 'unlockbuttons',
							desc = L['Unlock Buttons'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						cascadeplayer = {
							type = 'toggle',
							name = 'cascadeplayer',
							desc = L['Cascade Player Config'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						showpets = {
							type = 'toggle',
							name = 'showpets',
							desc = L['Show Pet Buttons'],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
						showtargettarget = {
							type = 'toggle',
							name = 'showtargettarget',
							desc = L["Show Target's Target"],
							get = BeneCast.GetProfileSettingsOption,
							set = BeneCast.SetProfileSettingsOption,
						},
					},
				},
				player = {
					type = 'group',
					name = L['Player'],
					args = { }, -- This will be filled as the spells load
				},
				pet = {
					type = 'group',
					name = L['Pet'],
					args = { }, -- This will be filled as the spells load
				},
				party = {
					type = 'group',
					name = L['Group Members'],
					args = { }, -- This will be filled as the spells load
				},
				partypet = {
					type = 'group',
					name = L['Group Pets'],
					args = { }, -- This will be filled as the spells load
				},
				other = {
					type = 'group',
					name = L['Non-Group Members'],
					args = { }, -- This will be filled as the spells load
				},
			},
        },
    },
};

defaultoptions.args.combat = _CopyTable(options.args.ooc, true);
defaultoptions.args.combat.name = 'combat';
defaultoptions.args.combat.desc = L['In Combat'];

-- Function to generate an optiontable from a single spell/item/...-name and a config
-- For now this is a toggle, but later it can be anything
function BeneCast:ActionToConfig(profileid, configname, shortgroupname, fullgroupname)

	local spelloption = {
		type = 'toggle',
		name = shortgroupname,
		arg = fullgroupname,
		get = function(info)
			local pfid = info[2];
			local configname = info[3];
			local config = BeneCastProfiles[pfid].config[configname];
			if config[info.arg] == nil then
				config[info.arg] = { enabled = false };
			end
			local setting = config[info.arg];
			return setting.enabled;
		end,
		set = function(info, val)
			local pfid = info[2];
			local configname = info[3];
			local mini, maxi;
			if BeneCastProfiles[pfid].cascadeplayer and configname == 'player' then
				mini = 1;
				maxi = table.max(BeneCast.Constants.Configs);
			else
				mini = _GetKeyForVal(BeneCast.Constants.Configs, configname);
				maxi = mini;
			end
			for i = mini, maxi do
				configname = BeneCast.Constants.Configs[i];
				local config = BeneCastProfiles[pfid].config[configname];
				if config[info.arg] == nil then
					config[info.arg] = { enabled = false };
				end
				local setting = config[info.arg];
				setting.enabled = val;
			end		
			BeneCast:UpdatePanelsForProfile(pfid);
		end,
	};
	return spelloption;

end

-- Function to generate entire spellconfig by the known spells-list
function BeneCast:GenerateSpellConfigFor(optiontable, profileid, configname)

	local configtable = optiontable.args.profiles.args[profileid].args[configname].args;
	local propergrp;
	_ClearTable(optionstable, true);
	
	-- First add assist and focus
	if configname ~= 'player' then
		configtable['assist'] = self:ActionToConfig(profileid, configname, BeneCast.Assist.group, BeneCast.Assist.group);
	end
	configtable['focus'] = self:ActionToConfig(profileid, configname, BeneCast.Focus.group, BeneCast.Focus.group);
	-- Then do spells
	for group, spell in pairs(wv.Spells) do
		if spell.target and BeneCast.Constants.TargetTypes[spell.target][configname] then
			propergrp = string.gsub(group, ' ', '');
			configtable[propergrp] = self:ActionToConfig(profileid, configname, propergrp, group);
		end
	end
	-- TODO: Items and macro's once the parsing of that is done

end

-- Function to do the above for ALL configs
function BeneCast:GenerateAllSpellConfigs(optiontable)

	for pfid, profile in pairs(BeneCastProfiles) do
		for i, configname in pairs(BeneCast.Constants.Configs) do
			self:GenerateSpellConfigFor(optiontable, pfid, configname);
		end
	end

end